package org.kantega.jexmec.jarfiles;

/*
 * Copyright 2011 Kantega AS
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


import org.kantega.jexmec.ClassLoaderProvider;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

/**
 * Convenient ClassLoaderProvider for adding plugins found from jar files in a given directory.
 * @see #addJarsInDirectory(java.io.File)
 */
public class JarFileClassLoaderProvider implements ClassLoaderProvider {
    private final List<ClassLoader> classLoaders = new ArrayList<ClassLoader>();
    private final File workDirectory;
    private final List<File>  filesAdded = new ArrayList<File>();
    private Registry registry;

    public JarFileClassLoaderProvider(File workDirectory) {
        this.workDirectory = workDirectory;
    }

    /**
     * Finds all *.jar files in the given directory and adds a {@link EmbeddedLibraryPluginClassLoader}
     * for each of them.
     * @param directory where to look for jar files
     */
    public void addJarsInDirectory(File directory) {

        if(!directory.isDirectory()) {
            return;
        }

        for(File file : directory.listFiles()) {
            if(file.isFile() && file.getName().toLowerCase().endsWith(".jar")) {
                filesAdded.add(file);
            }
        }
    }

    public void start(Registry registry, ClassLoader parentClassLoader) {
        this.registry = registry;

        List<ClassLoader> classLoaders = new ArrayList<ClassLoader>();
        for(File file : filesAdded) {
            classLoaders.add(new EmbeddedLibraryPluginClassLoader(file, parentClassLoader, workDirectory));
        }
        registry.add(classLoaders);
        this.classLoaders.addAll(classLoaders);

        File[] files = workDirectory.listFiles();
        if(files != null) {
            for(File file : files) {
                if(!filesAdded.contains(file)) {
                    deleteFile(file);
                }
            }
        }
    }

    public void stop() {
        registry.remove(classLoaders);
    }

    private void deleteFile(File file) {
        if(file.isDirectory()) {
            for(File child : file.listFiles()) {
                deleteFile(child);
            }
        }
        file.delete();
    }
}
